@@ -8,7 +8,7 @@
  * @short_description: disk label (PT) specific data and functions
  *
  * The fdisk_new_context() initializes all label drivers, and allocate
- * per-label specific data struct. This concept allows to store label specific
+ * per-label specific data struct. This concept can be used to store label specific
  * settings to the label driver independently on the currently active label
  * driver. Note that label struct cannot be deallocated, so there is no
  * reference counting for fdisk_label objects. All is destroyed by
@@ -269,7 +269,7 @@ int fdisk_write_disklabel(struct fdisk_context *cxt)
  *
  * Verifies the partition table.
  *
- * Returns: 0 on success, otherwise, a corresponding error.
+ * Returns: 0 on success, <1 runtime or option errors, >0 number of detected issues
  */
 int fdisk_verify_disklabel(struct fdisk_context *cxt)
 {
@@ -481,7 +481,27 @@ int fdisk_set_disklabel_id(struct fdisk_context *cxt)
 		return -ENOSYS;
 
 	DBG(CXT, ul_debugobj(cxt, "setting %s disk ID", cxt->label->name));
-	return cxt->label->op->set_id(cxt);
+	return cxt->label->op->set_id(cxt, NULL);
+}
+
+/**
+ * fdisk_set_disklabel_id_from_string
+ * @cxt: fdisk context
+ * @str: new Id
+ *
+ * Returns: 0 on success, otherwise, a corresponding error.
+ *
+ * Since: 2.36
+ */
+int fdisk_set_disklabel_id_from_string(struct fdisk_context *cxt, const char *str)
+{
+	if (!cxt || !cxt->label || !str)
+		return -EINVAL;
+	if (!cxt->label->op->set_id)
+		return -ENOSYS;
+
+	DBG(CXT, ul_debugobj(cxt, "setting %s disk ID from '%s'", cxt->label->name, str));
+	return cxt->label->op->set_id(cxt, str);
 }
 
 /**
