@@ -1764,7 +1764,7 @@ int mnt_context_prepare_srcpath(struct libmnt_context *cxt)
 {
 	const char *path = NULL;
 	struct libmnt_cache *cache;
-	const char *t, *v, *src;
+	const char *t, *v, *src, *type;
 	int rc = 0;
 	struct libmnt_ns *ns_old;
 
@@ -1786,6 +1786,11 @@ int mnt_context_prepare_srcpath(struct libmnt_context *cxt)
 	if (!src || mnt_fs_is_netfs(cxt->fs))
 		return 0;
 
+	/* ZFS source is always "dataset", not a real path */
+	type = mnt_fs_get_fstype(cxt->fs);
+	if (type && strcmp(type, "zfs") == 0)
+		return 0;
+
 	DBG(CXT, ul_debugobj(cxt, "srcpath '%s'", src));
 
 	ns_old = mnt_context_switch_target_ns(cxt);
@@ -2096,6 +2101,11 @@ int mnt_context_prepare_helper(struct libmnt_context *cxt, const char *name,
 	assert(cxt->fs);
 	assert((cxt->flags & MNT_FL_MOUNTFLAGS_MERGED));
 
+	if (cxt->helper) {
+		free(cxt->helper);
+		cxt->helper = NULL;
+	}
+
 	if (!type)
 		type = mnt_fs_get_fstype(cxt->fs);
 
