@@ -745,7 +745,7 @@ int mnt_optstr_apply_flags(char **optstr, unsigned long flags,
 	if (!optstr || !map)
 		return -EINVAL;
 
-	DBG(CXT, ul_debug("applying 0x%08lu flags to '%s'", flags, *optstr));
+	DBG(CXT, ul_debug("applying 0x%08lx flags to '%s'", flags, *optstr));
 
 	maps[0] = map;
 	next = *optstr;
@@ -983,7 +983,7 @@ int mnt_optstr_fix_uid(char **optstr, char *value, size_t valsz, char **next)
 	    (*(value + 7) == ',' || !*(value + 7)))
 		return set_uint_value(optstr, getuid(), value, end, next);
 
-	else if (!isdigit(*value)) {
+	if (!isdigit(*value)) {
 		uid_t id;
 		int rc;
 		char *p = strndup(value, valsz);
@@ -1031,7 +1031,7 @@ int mnt_optstr_fix_gid(char **optstr, char *value, size_t valsz, char **next)
 	    (*(value + 7) == ',' || !*(value + 7)))
 		return set_uint_value(optstr, getgid(), value, end, next);
 
-	else if (!isdigit(*value)) {
+	if (!isdigit(*value)) {
 		int rc;
 		gid_t id;
 		char *p = strndup(value, valsz);
@@ -1077,7 +1077,7 @@ int mnt_optstr_fix_user(char **optstr)
 	if (!username)
 		return -ENOMEM;
 
-	if (!ol.valsz || (ol.value && strncmp(ol.value, username, ol.valsz))) {
+	if (!ol.valsz || (ol.value && strncmp(ol.value, username, ol.valsz) != 0)) {
 		if (ol.valsz)
 			/* remove old value */
 			mnt_optstr_remove_option_at(optstr, ol.value, ol.end);
