@@ -68,6 +68,7 @@
 #include <syslog.h>
 
 #ifdef HAVE_LIBSYSTEMD
+# define SD_JOURNAL_SUPPRESS_LOCATION
 # include <systemd/sd-daemon.h>
 # include <systemd/sd-journal.h>
 #endif
@@ -217,6 +218,8 @@ static int pencode(char *s)
 	int facility, level;
 	char *separator;
 
+	assert(s);
+
 	separator = strchr(s, '.');
 	if (separator) {
 		*separator = '\0';
@@ -236,7 +239,7 @@ static int pencode(char *s)
 
 static int unix_socket(struct logger_ctl *ctl, const char *path, int *socket_type)
 {
-	int fd, i, type = -1;
+	int fd = -1, i, type = -1;
 	static struct sockaddr_un s_addr;	/* AF_UNIX address of local logger */
 
 	if (strlen(path) >= sizeof(s_addr.sun_path))
@@ -335,11 +338,11 @@ static int journald_entry(struct logger_ctl *ctl, FILE *fp)
 	struct iovec *iovec;
 	char *buf = NULL;
 	ssize_t sz;
-	int n, lines, vectors = 8, ret = 0;
+	int n, lines = 0, vectors = 8, ret = 0, msgline = -1;
 	size_t dummy = 0;
 
 	iovec = xmalloc(vectors * sizeof(struct iovec));
-	for (lines = 0; /* nothing */ ; lines++) {
+	while (1) {
 		buf = NULL;
 		sz = getline(&buf, &dummy, fp);
 		if (sz == -1 ||
@@ -347,6 +350,25 @@ static int journald_entry(struct logger_ctl *ctl, FILE *fp)
 			free(buf);
 			break;
 		}
+
+		if (strncmp(buf, "MESSAGE=", 8) == 0) {
+			if (msgline == -1)
+				msgline = lines;	/* remember the first message */
+			else {
+				char *p = xrealloc(iovec[msgline].iov_base,
+						   iovec[msgline].iov_len + sz - 8 + 2);
+
+				iovec[msgline].iov_base = p;
+				p += iovec[msgline].iov_len;
+				*p++ = '\n';
+				memcpy(p, buf + 8, sz - 8);
+				free(buf);
+
+				iovec[msgline].iov_len += sz - 8 + 1;
+				continue;
+			}
+		}
+
 		if (lines == vectors) {
 			vectors *= 2;
 			if (IOV_MAX < vectors)
@@ -355,6 +377,7 @@ static int journald_entry(struct logger_ctl *ctl, FILE *fp)
 		}
 		iovec[lines].iov_base = buf;
 		iovec[lines].iov_len = sz;
+		++lines;
 	}
 
 	if (!ctl->noact)
@@ -390,17 +413,17 @@ static char const *rfc3164_current_time(void)
 {
 	static char time[32];
 	struct timeval tv;
-	struct tm *tm;
+	struct tm tm;
 	static char const * const monthnames[] = {
 		"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug",
 		"Sep", "Oct", "Nov", "Dec"
 	};
 
 	logger_gettimeofday(&tv, NULL);
-	tm = localtime(&tv.tv_sec);
+	localtime_r(&tv.tv_sec, &tm);
 	snprintf(time, sizeof(time),"%s %2d %2.2d:%2.2d:%2.2d",
-		monthnames[tm->tm_mon], tm->tm_mday,
-		tm->tm_hour, tm->tm_min, tm->tm_sec);
+		monthnames[tm.tm_mon], tm.tm_mday,
+		tm.tm_hour, tm.tm_min, tm.tm_sec);
 	return time;
 }
 
@@ -494,7 +517,7 @@ static void write_output(struct logger_ctl *ctl, const char *const msg)
 		 * Let's do the same to be robust.    [kzak -- Oct 2017]
 		 *
 		 * MSG_NOSIGNAL is POSIX.1-2008 compatible, but it for example
-		 * no suported by apple-darwin15.6.0.
+		 * not supported by apple-darwin15.6.0.
 		 */
 #ifndef MSG_NOSIGNAL
 # define MSG_NOSIGNAL 0
@@ -752,13 +775,13 @@ static void syslog_rfc5424_header(struct logger_ctl *const ctl)
 
 	if (ctl->rfc5424_time) {
 		struct timeval tv;
-		struct tm *tm;
+		struct tm tm;
 
 		logger_gettimeofday(&tv, NULL);
-		if ((tm = localtime(&tv.tv_sec)) != NULL) {
+		if (localtime_r(&tv.tv_sec, &tm) != NULL) {
 			char fmt[64];
 			const size_t i = strftime(fmt, sizeof(fmt),
-						  "%Y-%m-%dT%H:%M:%S.%%06u%z ", tm);
+						  "%Y-%m-%dT%H:%M:%S.%%06u%z ", &tm);
 			/* patch TZ info to comply with RFC3339 (we left SP at end) */
 			fmt[i - 1] = fmt[i - 2];
 			fmt[i - 2] = fmt[i - 3];
@@ -1134,7 +1157,7 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	INIT_LIST_HEAD(&ctl.user_sds);
 	INIT_LIST_HEAD(&ctl.reserved_sds);
@@ -1191,11 +1214,6 @@ int main(int argc, char **argv)
 		case 'P':
 			ctl.port = optarg;
 			break;
-		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			exit(EXIT_SUCCESS);
-		case 'h':
-			usage();
 		case OPT_OCTET_COUNT:
 			ctl.octet_count = 1;
 			break;
@@ -1242,6 +1260,11 @@ int main(int argc, char **argv)
 				errx(EXIT_FAILURE, _("invalid structured data parameter: '%s'"), optarg);
 			add_structured_data_param(get_user_structured_data(&ctl), optarg);
 			break;
+
+		case 'V':
+			print_version(EXIT_SUCCESS);
+		case 'h':
+			usage();
 		default:
 			errtryhelp(EXIT_FAILURE);
 		}
@@ -1261,7 +1284,7 @@ int main(int argc, char **argv)
 	}
 #endif
 
-	/* user overwrites build-in SD-ELEMENT */
+	/* user overwrites built-in SD-ELEMENT */
 	if (has_structured_data_id(get_user_structured_data(&ctl), "timeQuality"))
 		ctl.rfc5424_tq = 0;
 
